@@ -7,7 +7,6 @@
 #' @param valData  SpatialPolygonsDataFrame or SpatialPointsDataFrame containing the validation locations (optional).
 #' @param responseCol Character or integer giving the column in \code{trainData}, which contains the response variable. Can be omitted, when \code{trainData} has only one column.
 #' @param nSamples Integer. Number of samples per land cover class.
-#' @param areaWeightedSampling Logical. If \code{TRUE} scales sample size per polygon area. The bigger the polygon the more samples are taken.
 #' @param polygonBasedCV Logical. If \code{TRUE} model tuning during cross-validation is conducted on a per-polygon basis. Use this to deal with overfitting issues. Does not affect training data supplied as SpatialPointsDataFrames.
 #' @param trainPartition Numeric. Partition (polygon based) of \code{trainData} that goes into the training data set between zero and one. Ignored if \code{valData} is provided.
 #' @param model Character. Which model to use. See \link[caret]{train} for options. Defaults to randomForest ('rf'). In addition to the standard caret models, a maximum likelihood classification is available via \code{model = 'mlc'}. 
@@ -30,7 +29,6 @@
 #' 
 #' \item Sample training coordinates. If \code{trainData} (and \code{valData} if present) are SpatialPolygonsDataFrames \code{superClass} will calculate the area per polygon and sample
 #' \code{nSamples} locations per class within these polygons. The number of samples per individual polygon scales with the polygon area, i.e. the bigger the polygon, the more samples.
-#' Setting \code{areaWeightedSampling = FALSE} will sample each polygon equally independent of its size.
 #' 
 #' \item Split training/validation	  
 #' If \code{valData} was provided (reccomended) the samples from these polygons will be held-out and not used for model fitting but only for validation. 
@@ -78,7 +76,7 @@
 #' horiz = TRUE,  bty = "n")
 #' par(olpar) # reset par
 superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
-        nSamples = 1000, areaWeightedSampling = TRUE, polygonBasedCV = FALSE, trainPartition = NULL,
+        nSamples = 1000, polygonBasedCV = FALSE, trainPartition = NULL,
         model = "rf", tuneLength = 3,  kfold = 5,
         minDist = 2,  mode = "classification", predict = TRUE, predType = "raw",
         filename = NULL, verbose,
@@ -86,7 +84,7 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
     # TODO: check applicability of raster:::.intersectExtent 
     # TODO: check for empty factor levels
     # TODO: consider splitting large polygons if there are few polygons in total
-    
+    seeds <- as.list(sample.int(kfold*tuneLength+2))
     if(!missing("verbose")) .initVerbose(verbose)
     verbose <- getOption("RStoolbox.verbose")
     ## Object types
@@ -132,9 +130,17 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
     if(mode == "classification" && is.numeric(trainData[[responseCol]])) {
         trainData[[responseCol]] <- as.factor(trainData[[responseCol]])       
     } 
+    
     ## Sanitize arguments (polygonBasedCV is only relevant for polygons)
     if(inherits(trainData, "SpatialPointsDataFrame") & polygonBasedCV) polygonBasedCV <- FALSE
     
+    ## Spit ellipsis into caret::trainControl and raster::writeRaster
+#    frmls_train <- names(formals(raster::writeRaster))
+#    args  <- c(list(...), method = method)
+#    args_trainControl  <- args[names(args) %in% frmls_train]
+#    args_writeRaster   <- args[!names(args) %in% frmls_train]
+#    args_writeRaster$filename <- if(length(classes) == 1) filename else NULL ## write raster here already during predict if only one layer is output
+#    
     
     ## Split into training and validation data (polygon basis)
     if(is.null(valData) & !is.null(trainPartition)){
@@ -160,58 +166,32 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
     }
     
     
-    ## Check for and deal with overlapping training & validation data                 
-    if(trainDataType == "points" & minDist != 0) dummy <- gBuffer(trainData, width = res(img)[1]*minDist, byid = TRUE)  
-    
-    ## We need this cumbersome any(gDisjoint(byid=T)) action to circumvent TopologyExceptions
-    #if(!is.null(valData) && any(!gDisjoint(trainData, valData, byid =T)) || exists("dummy") &&  any(gIntersects(valData, dummy, byid=T))){
     if(identical(trainData, valData)) stop("trainData is the same as valData")
-    if(trainDataType == "polygons"){
-        ## Clip validation data to training data + 2 pixel buffer 
-        #dissolve(gUnionCascaded(trainData, trainData[[responseCol]]))        
-        #inter <- gIntersection(valData, trainData, byid = TRUE) ## again both steps needed to deal with poor poly data potentially arising from manually digitizing training areas
+    if(!is.null(valData)){
         
-        trainBuff <- if(minDist > 0) gBuffer(trainData, width = res(img)[1]*minDist) else gUnionCascaded(trainData)
-        clip  <- gDifference(valData, trainBuff, byid = TRUE)
-        if(is.null(clip)) stop("After clipping valData to trainData+minDist*pix buffer no validation polygons remain. Please provide non-overlapping trainData and valData.")
-        
-        
-#            if(inherits(inter, "SpatialCollections")) {
-#                ## This happens when polygons share borders
-#                ## Make lines or points polygons 
-#                l2poly <- if(!is.null(inter@lineobj))  gBuffer(inter@lineobj, width = res(img)[1]*1e-5, byid = TRUE) else NULL
-#                p2poly <- if(!is.null(inter@pointobj)) gBuffer(inter@pointobj, width = res(img)[1]*1e-5, byid = TRUE) else NULL
-#                ## Merge all polygons into single list
-#                plist <- list(l2poly, p2poly, inter@polyobj)
-#                ## Remove non-matching classes
-#                plist[vapply(plist, is.null, logical(1))] <- NULL
-#                inter <- do.call("rbind", c(plist, makeUniqueIDs = TRUE))      
-#                
-#            } 
-        
-#            inter <- gUnionCascaded(inter)
-#            
-#            ## Buffer train polygons by mindist and clip valData with it
-#            if(minDist != 0) inter <- gBuffer(inter, width = res(img)[1] * minDist)
-#            clip  <- gDifference(valData, inter, byid = TRUE)
-#            if(is.null(clip)) stop("After clipping valData to trainData+minDist*pix buffer no validation polygons remain. Please provide non-overlapping trainData and valData.")
-#            
-        ## Add class labels back to polygons
-        classVec <- data.frame(x = over(clip, valData)[[responseCol]])
-        valData <- as(clip, "SpatialPolygonsDataFrame")
-        valData@data <- classVec 
-        colnames(valData@data) <- responseCol
-        
-    } else { 
-        if(!exists("dummy")) dummy <- trainData 
-        inter <- gIntersects(valData, dummy, byid = T)
-        if(any(inter)){            
-            inter <- colnames(inter)[which(inter, arr.ind = TRUE)[,2]]
-            valData <- valData[!rownames(valData@data) %in% inter,]
-        }
-    }     
-    
-    ## Creade hold out indices on polygon level
+        if(trainDataType == "polygons" ){
+            ## Clip validation data to training data + 2 pixel buffer 
+            trainBuff <- if(minDist > 0) .omniBuffer(trainData, minDist = minDist, img = img) else gUnionCascaded(trainData)
+            clip      <- gDifference(valData, trainBuff, byid = TRUE)
+            if(is.null(clip)) stop("After clipping valData to trainData+minDist*pix buffer no validation polygons remain. Please provide non-overlapping trainData and valData.")
+            
+            ## Add class labels back to polygons
+            classVec <- data.frame(x = over(clip, valData)[[responseCol]])
+            valData  <- as(clip, "SpatialPolygonsDataFrame")
+            valData@data <- classVec 
+            colnames(valData@data) <- responseCol
+            
+        } else { 
+            ## Check for and deal with overlapping training & validation data                 
+            dummy <- if(minDist != 0) .omniBuffer(trainData, minDist = minDist, img = img)  else trainData
+            inter <- gIntersects(valData, dummy, byid = T)
+            if(any(inter)){            
+                inter <- colnames(inter)[which(inter, arr.ind = TRUE)[,2]]
+                valData <- valData[!rownames(valData@data) %in% inter,]
+            }
+        }     
+    }
+    ## Create hold out indices on polygon level
     if(polygonBasedCV){
         folds        <- createFolds(trainData@data[[responseCol]], k = kfold)
         names(folds) <- NULL
@@ -224,57 +204,14 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
     
     ## Calculate area weighted number of samples per polygon
     ## we'll end up with n > nSamples, but make sure to sample each polygon at least once
-    .samplePixels <- function(SHAPE, RASTER, foldCol = NULL, trainCells = NULL){
-        if(trainDataType == "polygons"){
-            if (areaWeightedSampling){
-                if(is.projected(SHAPE)){
-                    SHAPE[["area"]] <- gArea(SHAPE, byid = TRUE)
-                } else {
-                    SHAPE[["area"]] <- areaPolygon(SHAPE)		
-                }
-            } else {
-                SHAPE[["area"]] <- 1
-            }
-            
-            ## Calculate optimal nSamples per class
-            SHAPE@data[["order"]] <- 1:nrow(SHAPE) 		
-            weights <- ddply(SHAPE@data, .variables = responseCol, .fun = here(mutate), nSamplesClass = ceiling(nSamples * area / sum(area)))
-            SHAPE@data <- weights[order(weights$order),]
-            
-            ## Get random coordinates within polygons
-            # xy  <- .parXapply(X = seq_along(SHAPE), XFUN ="lapply", FUN=function(i_poly, shap = SHAPE){
-            #            sps <- spsample(shap[i_poly, ], type = "random", n = shap@data[i_poly,"nSamplesClass"], iter = 20)
-            #           return(sps)
-            #      } , envir = environment())
-            xy  <- lapply(seq_along(SHAPE), FUN=function(i_poly, shap = SHAPE){
-                        spsample(shap[i_poly, ], type = "random", n = shap@data[i_poly,"nSamplesClass"], iter = 20)
-                    })
-            xy  <- do.call("rbind", xy)
-        } else {
-            xy <- SHAPE
-        }
-        
-        ## Extract response and predictors and combine in final training set
-        dataSet <- data.frame(
-                if(trainDataType == "polygons") over(x = xy, y = SHAPE)[c(responseCol,foldCol)] else SHAPE[[responseCol]],
-                extract(RASTER, xy, cellnumbers = TRUE))
-        
-        ## Discard duplicate cells
-        dubs 	<- !duplicated(dataSet[,"cells"]) & complete.cases(dataSet) & !dataSet[,"cells"] %in% trainCells
-        dataSet <- dataSet[dubs,]
-        #dataSet <- dataSet[complete.cases(dataSet),]
-        # dataSet$cells <- NULL
-        colnames(dataSet)[1] <- "response"
-        list(dataSet[,setdiff(colnames(dataSet), "cells")], cells = dataSet$cells)
-    }
-    
     .vMessage("Begin sampling training data")
-    dataList  <- .samplePixels(SHAPE = trainData, RASTER=img, foldCol=foldCol)
+    dataList  <- .samplePixels(SHAPE = trainData, RASTER=img, responseCol = responseCol, nSamples = nSamples)
     dataSet   <- dataList[[1]]
     if(polygonBasedCV) {
         indexOut <- dataSet[[foldCol]]
         dataSet[[foldCol]] <- NULL
     }
+    
     ## Unique classes
     if(mode == "classification"){   
         if(!is.factor(dataSet$response)) dataSet$response <- as.factor(dataSet$response)
@@ -297,12 +234,12 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
     .registerDoParallel()
     indexIn <- if(polygonBasedCV) lapply(1:kfold, function(x) which(x != indexOut)) 
     if(model == "mlc") model = mlcCaret
-    
+    set.seed(seeds[[1]])
     caretModel 	<- train(response ~ ., data = dataSet, method = model, tuneLength = tuneLength, 
-            trControl = trainControl(method = "cv", number = kfold, index = indexIn), ...)   
+            trControl = trainControl(method = "cv", number = kfold, index = indexIn, savePredictions = "final"), ...)   
     modelFit <- getTrainPerf(caretModel)
-    dataType <- NULL
     
+    dataType <- NULL  
     if(mode == "classification") {
         ## Don't know whether we need this, who would be crazy enough to do more than 255 classes...
         dataType <- if(length(classes) < 255) "INT1U" else "INT2U"
@@ -318,7 +255,7 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
         wrArgs          <- list(filename = filename, progress = progress, datatype = dataType, overwrite = overwrite)
         wrArgs$filename <- filename ## remove filename from args if is.null(filename) --> standard writeRaster handling applies
         if(predType == "prob") {
-            ddd<- predict(caretModel, dataSet[1:2,-1,drop=FALSE], type="prob")
+            ddd     <- predict(caretModel, dataSet[1:2,-1,drop=FALSE], type="prob")
             probInd <- 1:ncol(ddd)
         } else {
             probInd <- 1
@@ -332,22 +269,26 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
     
     ## VALIDATION ########################
     if(!is.null(valData)){
-        if(predict){
-            valiSet  <- .samplePixels(valData, spatPred, trainCells = dataList[[2]])[[1]]
+        nSamplesV <- unlist(Map("max", nSamples, 500))
+        
+        if(predict & (predType == "raw")){
+            set.seed(seeds[[kfold+2]])
+            valiSet  <- .samplePixels(valData, spatPred, responseCol = responseCol, nSamples = nSamplesV,  trainCells = dataList[[2]])[[1]]
             colnames(valiSet) <- c("reference", "prediction")
         } else {
-            val <- .samplePixels(valData, img, trainCells = dataList[[2]])[[1]]
+            set.seed(seeds[[kfold+2]])
+            val <- .samplePixels(valData, img, responseCol = responseCol, nSamples = nSamplesV, trainCells = dataList[[2]])[[1]]
             pred <- predict(caretModel, val[,-1])
             valiSet <- data.frame(reference = val[,1], prediction = pred)
         }
         
         if(mode == "classification"){
-            if(!is.factor(valiSet$reference))  valiSet$reference <- factor(valiSet$reference, levels = levels(classes))
+            if(!is.factor(valiSet$reference))  valiSet$reference  <- factor(valiSet$reference, levels = levels(classes))
             if(is.numeric(valiSet$prediction)) valiSet$prediction <- factor(levels(classes)[valiSet$prediction], levels = levels(classes))
             validation <- confusionMatrix(data = valiSet$prediction, reference = valiSet$reference)              
         } else {
             valiSet$residuals <- valiSet$reference - valiSet$prediction
-            validation <-  data.frame(rmse = RMSE(valiSet$prediction, valiSet$reference), rsquared = R2(valiSet$prediction, valiSet$reference))   
+            validation <-  data.frame(RMSE = RMSE(valiSet$prediction, valiSet$reference), Rsquared = R2(valiSet$prediction, valiSet$reference))   
         }
         validation <- list(performance = validation, validationSet = valiSet)
     } else {
@@ -370,6 +311,77 @@ superClass <- function(img, trainData, valData = NULL, responseCol = NULL,
 }
 
 
+.samplePixels <- function(SHAPE, RASTER, responseCol, trainCells = NULL, nSamples, maxnpix = FALSE){
+    
+    if(inherits(SHAPE, "SpatialPolygons")){					
+        #cells <- cellFromPolygon(RASTER, SHAPE),
+        r    <- raster(RASTER)	
+        buff <- max(res(RASTER))
+        polyCells <- .parXapply(X = 1:nrow(SHAPE), XFUN="lapply", FUN= function(i) { 
+                    rc <- crop(r, extent(SHAPE[i,]) + buff)
+                    rc <- rasterize(SHAPE[i,], rc, fun = "first", silent=TRUE)
+                    xy <- rasterToPoints(rc)[,-3,drop=FALSE]			
+                    cellFromXY(RASTER, xy)
+                }, envir = environment()) 
+        
+        area  <- lapply(polyCells, length) 
+        resp  <- SHAPE[[responseCol]]
+        uresp <- unique(resp)
+        totalarea <- vapply(uresp, function(xi) sum(unlist(area[resp == xi])), numeric(1))			
+        if(maxnpix) nSamples  <- min(totalarea)
+        dataSet   <- lapply(seq_along(polyCells), function(xi) {
+                    class <- resp[[xi]]
+                    ns <- min(ceiling(nSamples * area[[xi]] / totalarea[which(uresp == resp[[xi]])]), area[[xi]] )
+                    if(ns > 0) data.frame(response = class, cells = sample(polyCells[[xi]], ns)) else NULL
+                })	
+        dataSet <- do.call("rbind", dataSet)
+        dataSet <- cbind(dataSet, RASTER[dataSet[,"cells"]])        
+    } else {
+        dataSet <- data.frame(response = SHAPE[[responseCol]], extract(RASTER, SHAPE, cell=TRUE))
+    }
+    
+    ## Discard duplicate cells
+    dubs 	<- !duplicated(dataSet[,"cells"]) & complete.cases(dataSet) & !dataSet[,"cells"] %in% trainCells
+    dataSet <- dataSet[dubs,]
+    list(dataSet[,setdiff(colnames(dataSet), "cells")], cells = dataSet$cells)
+}
+
+
+#' Calculate buffers around trainingData regardless if they are projected or not
+#' 
+#' for unprojected data_ projects the data to azimuthal equidistant projection, calculates the buffer and reprojects
+#' for projected data only buffers
+#' @param x Polygons or Points
+#' @param minDist buffer distance (in pixels)
+#' @param img raster to query resolution and calculate the buffer in m
+#' @noRd 
+#' @keywords internal
+.omniBuffer <- function(x, minDist, img){
+    
+    if(!is.projected(x)){
+        crx  <- projection(x)
+        ## Project to azimuthal equidistant centered on training data center
+        exc  <- .extentCenter(extent(x))
+        aeqd <- paste0("+proj=aeqd +lat_0=", exc[2]," +lon_0=",exc[1], " +x_0=0 +y_0=0 +ellps=WGS84")
+        x    <- spTransform(x, CRS(aeqd))
+        
+        ## Get raster resolution in projected coordinates
+        rastRes <- max(res(projectRaster(img[1,,drop=FALSE], crs = aeqd)))
+        
+        ## Buffer
+        x <- gBuffer(x, width = minDist * rastRes)
+        
+        ## Transform back
+        return(spTransform(x, crx))
+    } else {
+        return(gBuffer(x, width = minDist * max(res(img))))
+    }               
+}
+
+
+
+
+
 #' Predict a raster map based on a superClass model fit.
 #' 
 #' useful to separate model fitting from spatial prediction, which can take some time.
@@ -399,7 +411,9 @@ predict.superClass <- function(object, img, predType = "raw", filename = NULL, d
     wrArgs$filename <- filename ## remove filename from args if is.null(filename) --> standard writeRaster handling applies
     
     if(predType == "prob") {
-        ddd <- predict(model, img[1:2,], type="prob")
+        py<-img[1:2]
+        py[]<-1
+        ddd <- predict(model, py, type="prob")
         probInd <- 1:ncol(ddd)
     } else {
         probInd <- 1
@@ -419,4 +433,3 @@ print.superClass <- function(x,...){
     cat("$map\n")
     show(x$map)
 }
-
